@@ -1,23 +1,19 @@
 /***************************************************************************/
 /*									   */
 /* ftplib.c - callable ftp access routines				   */
-/* Copyright (C) 1996-2001 Thomas Pfau, pfau@nbpfaus.net		   */
+/* Copyright (C) 1996-2001, 2013 Thomas Pfau, tfpfau@gmail.com		   */
 /*	1407 Thomas Ave, North Brunswick, NJ, 08902			   */
 /*									   */
-/* This library is free software; you can redistribute it and/or	   */
-/* modify it under the terms of the GNU Library General Public		   */
-/* License as published by the Free Software Foundation; either		   */
-/* version 2 of the License, or (at your option) any later version.	   */
+/* This library is free software.  You can redistribute it and/or	   */
+/* modify it under the terms of the Artistic License 2.0.		   */
 /* 									   */
 /* This library is distributed in the hope that it will be useful,	   */
 /* but WITHOUT ANY WARRANTY; without even the implied warranty of	   */
-/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU	   */
-/* Library General Public License for more details.			   */
+/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	   */
+/* Artistic License 2.0 for more details.				   */
 /* 									   */
-/* You should have received a copy of the GNU Library General Public	   */
-/* License along with this progam; if not, write to the			   */
-/* Free Software Foundation, Inc., 59 Temple Place - Suite 330,		   */
-/* Boston, MA 02111-1307, USA.						   */
+/* See the file LICENSE or 						   */
+/* http://www.perlfoundation.org/artistic_license_2_0			   */
 /* 									   */
 /***************************************************************************/
 
@@ -48,10 +44,21 @@
 #elif defined(_WIN32)
 #include <winsock.h>
 #endif
+#if defined(__APPLE__)
+#undef _REENTRANT
+#endif
 
 #define BUILDING_LIBRARY
 #include "ftplib.h"
 
+#if defined(__UINT64_MAX) && !defined(PRIu64)
+#if ULONG_MAX == __UINT32_MAX
+#define PRIu64 "llu"
+#else
+#define PRIu64 "lu"
+#endif
+#endif
+
 #if defined(_WIN32)
 #define SETSOCKOPT_OPTVAL_TYPE (const char *)
 #else
@@ -59,6 +66,8 @@
 #endif
 
 #define FTPLIB_BUFSIZ 8192
+#define RESPONSE_BUFSIZ 1024
+#define TMP_BUFSIZ 1024
 #define ACCEPT_TIMEOUT 30
 
 #define FTPLIB_CONTROL 0
@@ -81,20 +90,59 @@ struct NetBuf {
     struct timeval idletime;
     FtpCallback idlecb;
     void *idlearg;
-    int xfered;
-    int cbbytes;
-    int xfered1;
-    char response[256];
+    unsigned long int xfered;
+    unsigned long int cbbytes;
+    unsigned long int xfered1;
+    char response[RESPONSE_BUFSIZ];
 };
 
 static char *version =
-    "ftplib Release 3.1-1 11/6/03, copyright 1996-2003 Thomas Pfau";
+    "ftplib Release 4.0 07-Jun-2013, copyright 1996-2003, 2013 Thomas Pfau";
 
 GLOBALDEF int ftplib_debug = 0;
 
 #if defined(__unix__) || defined(VMS)
-#define net_read read
-#define net_write write
+int net_read(int fd, char *buf, size_t len)
+{
+    while ( 1 )
+    {
+	int c = read(fd, buf, len);
+	if ( c == -1 )
+	{
+	    if ( errno != EINTR && errno != EAGAIN )
+		return -1;
+	}
+	else
+	{
+	    return c;
+	}
+    }
+}
+
+int net_write(int fd, const char *buf, size_t len)
+{
+    int done = 0;
+    while ( len > 0 )
+    {
+	int c = write( fd, buf, len );
+	if ( c == -1 )
+	{
+	    if ( errno != EINTR && errno != EAGAIN )
+		return -1;
+	}
+	else if ( c == 0 )
+	{
+	    return done;
+	}
+	else
+	{
+	    buf += c;
+	    done += c;
+	    len -= c;
+	}
+    }
+    return done;
+}
 #define net_close close
 #elif defined(_WIN32)
 #define net_read(x,y,z) recv(x,y,z,0)
@@ -239,7 +287,8 @@ static int readline(char *buf,int max,netbuf *ctl)
 	    return retval;
     	if ((x = net_read(ctl->handle,ctl->cput,ctl->cleft)) == -1)
     	{
-	    perror("read");
+	    if (ftplib_debug)
+		perror("read");
 	    retval = -1;
 	    break;
     	}
@@ -258,10 +307,11 @@ static int readline(char *buf,int max,netbuf *ctl)
  *
  * return -1 on error or bytecount
  */
-static int writeline(char *buf, int len, netbuf *nData)
+static int writeline(const char *buf, int len, netbuf *nData)
 {
     int x, nb=0, w;
-    char *ubp = buf, *nbp;
+    const char *ubp = buf;
+    char *nbp;
     char lc=0;
 
     if (nData->dir != FTPLIB_WRITE)
@@ -278,7 +328,8 @@ static int writeline(char *buf, int len, netbuf *nData)
 		w = net_write(nData->handle, nbp, FTPLIB_BUFSIZ);
 		if (w != FTPLIB_BUFSIZ)
 		{
-		    printf("net_write(1) returned %d, errno = %d\n", w, errno);
+		    if (ftplib_debug)
+			printf("net_write(1) returned %d, errno = %d\n", w, errno);
 		    return(-1);
 		}
 		nb = 0;
@@ -292,7 +343,8 @@ static int writeline(char *buf, int len, netbuf *nData)
 	    w = net_write(nData->handle, nbp, FTPLIB_BUFSIZ);
 	    if (w != FTPLIB_BUFSIZ)
 	    {
-		printf("net_write(2) returned %d, errno = %d\n", w, errno);
+		if (ftplib_debug)
+		    printf("net_write(2) returned %d, errno = %d\n", w, errno);
 		return(-1);
 	    }
 	    nb = 0;
@@ -306,7 +358,8 @@ static int writeline(char *buf, int len, netbuf *nData)
 	w = net_write(nData->handle, nbp, nb);
 	if (w != nb)
 	{
-	    printf("net_write(3) returned %d, errno = %d\n", w, errno);
+	    if (ftplib_debug)
+		printf("net_write(3) returned %d, errno = %d\n", w, errno);
 	    return(-1);
 	}
     }
@@ -322,9 +375,10 @@ static int writeline(char *buf, int len, netbuf *nData)
 static int readresp(char c, netbuf *nControl)
 {
     char match[5];
-    if (readline(nControl->response,256,nControl) == -1)
+    if (readline(nControl->response,RESPONSE_BUFSIZ,nControl) == -1)
     {
-	perror("Control socket read failed");
+	if (ftplib_debug)
+	    perror("Control socket read failed");
 	return 0;
     }
     if (ftplib_debug > 1)
@@ -336,9 +390,10 @@ static int readresp(char c, netbuf *nControl)
 	match[4] = '\0';
 	do
 	{
-	    if (readline(nControl->response,256,nControl) == -1)
+	    if (readline(nControl->response,RESPONSE_BUFSIZ,nControl) == -1)
 	    {
-		perror("Control socket read failed");
+		if (ftplib_debug)
+		    perror("Control socket read failed");
 		return 0;
 	    }
 	    if (ftplib_debug > 1)
@@ -385,8 +440,6 @@ GLOBALDEF int FtpConnect(const char *host, netbuf **nControl)
 {
     int sControl;
     struct sockaddr_in sin;
-    struct hostent *phe;
-    struct servent *pse;
     int on=1;
     netbuf *ctrl;
     char *lhost;
@@ -397,69 +450,98 @@ GLOBALDEF int FtpConnect(const char *host, netbuf **nControl)
     lhost = strdup(host);
     pnum = strchr(lhost,':');
     if (pnum == NULL)
-    {
-#if defined(VMS)
-    	sin.sin_port = htons(21);
-#else
-    	if ((pse = getservbyname("ftp","tcp")) == NULL)
-    	{
-	    perror("getservbyname");
-	    return 0;
-    	}
-    	sin.sin_port = pse->s_port;
-#endif
-    }
+	pnum = "ftp";
+    else
+	*pnum++ = '\0';
+    if (isdigit(*pnum))
+	sin.sin_port = htons(atoi(pnum));
     else
     {
-	*pnum++ = '\0';
-	if (isdigit(*pnum))
-	    sin.sin_port = htons(atoi(pnum));
-	else
+	struct servent *pse;
+#if _REENTRANT
+	struct servent se;
+	char tmpbuf[TMP_BUFSIZ];
+	int i;
+	if ( ( i = getservbyname_r(pnum,"tcp",&se,tmpbuf,TMP_BUFSIZ,&pse) ) != 0 )
 	{
-	    pse = getservbyname(pnum,"tcp");
-	    sin.sin_port = pse->s_port;
+	    errno = i;
+	    if ( ftplib_debug )
+		perror("getservbyname_r");
+	    free(lhost);
+	    return 0;
 	}
+#else
+	if ((pse = getservbyname(pnum,"tcp") ) == NULL )
+	{
+	    if ( ftplib_debug )
+		perror("getservbyname");
+	    free(lhost);
+	    return 0;
+	}
+#endif
+	sin.sin_port = pse->s_port;
     }
-    if ((sin.sin_addr.s_addr = inet_addr(lhost)) == -1)
+    if ((sin.sin_addr.s_addr = inet_addr(lhost)) == INADDR_NONE)
     {
+	struct hostent *phe;
+#ifdef _REENTRANT
+	struct hostent he;
+	char tmpbuf[TMP_BUFSIZ];
+	int i, herr;
+	if ( ( i = gethostbyname_r( lhost, &he, tmpbuf, TMP_BUFSIZ, &phe, &herr ) ) != 0 )
+	{
+	    if ( ftplib_debug )
+		fprintf(stderr, "gethostbyname: %s\n", hstrerror(herr));
+	    free(lhost);
+	    return 0;
+	}
+#else
     	if ((phe = gethostbyname(lhost)) == NULL)
     	{
-	    perror("gethostbyname");
+	    if (ftplib_debug)
+		fprintf(stderr, "gethostbyname: %s\n", hstrerror(h_errno));
+	    free(lhost);
 	    return 0;
     	}
+#endif
     	memcpy((char *)&sin.sin_addr, phe->h_addr, phe->h_length);
     }
     free(lhost);
     sControl = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
     if (sControl == -1)
     {
-	perror("socket");
+	if (ftplib_debug)
+	    perror("socket");
 	return 0;
     }
     if (setsockopt(sControl,SOL_SOCKET,SO_REUSEADDR,
 		   SETSOCKOPT_OPTVAL_TYPE &on, sizeof(on)) == -1)
     {
-	perror("setsockopt");
+	if (ftplib_debug)
+	    perror("setsockopt");
 	net_close(sControl);
 	return 0;
     }
     if (connect(sControl, (struct sockaddr *)&sin, sizeof(sin)) == -1)
     {
-	perror("connect");
+	if (ftplib_debug)
+	    perror("connect");
 	net_close(sControl);
 	return 0;
     }
     ctrl = calloc(1,sizeof(netbuf));
     if (ctrl == NULL)
     {
-	perror("calloc");
+	if (ftplib_debug)
+	    perror("calloc");
 	net_close(sControl);
 	return 0;
     }
     ctrl->buf = malloc(FTPLIB_BUFSIZ);
     if (ctrl->buf == NULL)
     {
-	perror("calloc");
+	if (ftplib_debug)
+	    perror("calloc");
 	net_close(sControl);
 	free(ctrl);
 	return 0;
@@ -485,6 +567,24 @@ GLOBALDEF int FtpConnect(const char *host, netbuf **nControl)
     return 1;
 }
 
+GLOBALDEF int FtpSetCallback(const FtpCallbackOptions *opt, netbuf *nControl)
+{
+    nControl->idlecb = opt->cbFunc;
+    nControl->idlearg = opt->cbArg;
+    nControl->idletime.tv_sec = opt->idleTime / 1000;
+    nControl->idletime.tv_usec = (opt->idleTime % 1000) * 1000;
+    nControl->cbbytes = opt->bytesXferred;
+    return 1;
+}
+GLOBALDEF int FtpClearCallback(netbuf *nControl)
+{
+    nControl->idlecb = NULL;
+    nControl->idlearg = NULL;
+    nControl->idletime.tv_sec = 0;
+    nControl->idletime.tv_usec = 0;
+    nControl->cbbytes = 0;
+    return 1;
+}
 /*
  * FtpOptions - change connection options
  *
@@ -532,7 +632,7 @@ GLOBALDEF int FtpOptions(int opt, long val, netbuf *nControl)
  */
 static int FtpSendCmd(const char *cmd, char expresp, netbuf *nControl)
 {
-    char buf[256];
+    char buf[TMP_BUFSIZ];
     if (nControl->dir != FTPLIB_CONTROL)
 	return 0;
     if (ftplib_debug > 2)
@@ -542,7 +642,8 @@ static int FtpSendCmd(const char *cmd, char expresp, netbuf *nControl)
     sprintf(buf,"%s\r\n",cmd);
     if (net_write(nControl->handle,buf,strlen(buf)) <= 0)
     {
-	perror("write");
+	if (ftplib_debug)
+	    perror("write");
 	return 0;
     }
     return readresp(expresp, nControl);
@@ -589,7 +690,7 @@ static int FtpOpenPort(netbuf *nControl, netbuf **nData, int mode, int dir)
     netbuf *ctrl;
     char *cp;
     unsigned int v[6];
-    char buf[256];
+    char buf[TMP_BUFSIZ];
 
     if (nControl->dir != FTPLIB_CONTROL)
 	return -1;
@@ -626,27 +727,31 @@ static int FtpOpenPort(netbuf *nControl, netbuf **nData, int mode, int dir)
     {
 	if (getsockname(nControl->handle, &sin.sa, &l) < 0)
 	{
-	    perror("getsockname");
-	    return 0;
+	    if (ftplib_debug)
+		perror("getsockname");
+	    return -1;
 	}
     }
     sData = socket(PF_INET,SOCK_STREAM,IPPROTO_TCP);
     if (sData == -1)
     {
-	perror("socket");
+	if (ftplib_debug)
+	    perror("socket");
 	return -1;
     }
     if (setsockopt(sData,SOL_SOCKET,SO_REUSEADDR,
 		   SETSOCKOPT_OPTVAL_TYPE &on,sizeof(on)) == -1)
     {
-	perror("setsockopt");
+	if (ftplib_debug)
+	    perror("setsockopt");
 	net_close(sData);
 	return -1;
     }
     if (setsockopt(sData,SOL_SOCKET,SO_LINGER,
 		   SETSOCKOPT_OPTVAL_TYPE &lng,sizeof(lng)) == -1)
     {
-	perror("setsockopt");
+	if (ftplib_debug)
+	    perror("setsockopt");
 	net_close(sData);
 	return -1;
     }
@@ -654,7 +759,8 @@ static int FtpOpenPort(netbuf *nControl, netbuf **nData, int mode, int dir)
     {
 	if (connect(sData, &sin.sa, sizeof(sin.sa)) == -1)
 	{
-	    perror("connect");
+	    if (ftplib_debug)
+		perror("connect");
 	    net_close(sData);
 	    return -1;
 	}
@@ -664,18 +770,20 @@ static int FtpOpenPort(netbuf *nControl, netbuf **nData, int mode, int dir)
 	sin.in.sin_port = 0;
 	if (bind(sData, &sin.sa, sizeof(sin)) == -1)
 	{
-	    perror("bind");
+	    if (ftplib_debug)
+		perror("bind");
 	    net_close(sData);
-	    return 0;
+	    return -1;
 	}
 	if (listen(sData, 1) < 0)
 	{
-	    perror("listen");
+	    if (ftplib_debug)
+		perror("listen");
 	    net_close(sData);
-	    return 0;
+	    return -1;
 	}
 	if (getsockname(sData, &sin.sa, &l) < 0)
-	    return 0;
+	    return -1;
 	sprintf(buf, "PORT %d,%d,%d,%d,%d,%d",
 		(unsigned char) sin.sa.sa_data[2],
 		(unsigned char) sin.sa.sa_data[3],
@@ -686,19 +794,21 @@ static int FtpOpenPort(netbuf *nControl, netbuf **nData, int mode, int dir)
 	if (!FtpSendCmd(buf,'2',nControl))
 	{
 	    net_close(sData);
-	    return 0;
+	    return -1;
 	}
     }
     ctrl = calloc(1,sizeof(netbuf));
     if (ctrl == NULL)
     {
-	perror("calloc");
+	if (ftplib_debug)
+	    perror("calloc");
 	net_close(sData);
 	return -1;
     }
     if ((mode == 'A') && ((ctrl->buf = malloc(FTPLIB_BUFSIZ)) == NULL))
     {
-	perror("calloc");
+	if (ftplib_debug)
+	    perror("calloc");
 	net_close(sData);
 	free(ctrl);
 	return -1;
@@ -797,7 +907,7 @@ static int FtpAcceptConnection(netbuf *nData, netbuf *nControl)
 GLOBALDEF int FtpAccess(const char *path, int typ, int mode, netbuf *nControl,
     netbuf **nData)
 {
-    char buf[256];
+    char buf[TMP_BUFSIZ];
     int dir;
     if ((path == NULL) &&
         ((typ == FTPLIB_FILE_WRITE) || (typ == FTPLIB_FILE_READ)))
@@ -835,7 +945,7 @@ GLOBALDEF int FtpAccess(const char *path, int typ, int mode, netbuf *nControl,
     {
         int i = strlen(buf);
         buf[i++] = ' ';
-        if ((strlen(path) + i) >= sizeof(buf))
+        if ((strlen(path) + i + 1) >= sizeof(buf))
             return 0;
         strcpy(&buf[i],path);
     }
@@ -898,7 +1008,7 @@ GLOBALDEF int FtpRead(void *buf, int max, netbuf *nData)
 /*
  * FtpWrite - write to a data connection
  */
-GLOBALDEF int FtpWrite(void *buf, int len, netbuf *nData)
+GLOBALDEF int FtpWrite(const void *buf, int len, netbuf *nData)
 {
     int i;
     if (nData->dir != FTPLIB_WRITE)
@@ -944,9 +1054,9 @@ GLOBALDEF int FtpClose(netbuf *nData)
 	net_close(nData->handle);
 	ctrl = nData->ctrl;
 	free(nData);
-	if (ctrl)
+	ctrl->data = NULL;
+	if (ctrl && ctrl->response[0] != '4' && ctrl->response[0] != 5)
 	{
-	    ctrl->data = NULL;
 	    return(readresp('2', ctrl));
 	}
 	return 1;
@@ -954,7 +1064,7 @@ GLOBALDEF int FtpClose(netbuf *nData)
 	if (nData->data)
 	{
 	    nData->ctrl = NULL;
-	    FtpClose(nData);
+	    FtpClose(nData->data);
 	}
 	net_close(nData->handle);
 	free(nData);
@@ -970,7 +1080,7 @@ GLOBALDEF int FtpClose(netbuf *nData)
  */
 GLOBALDEF int FtpSite(const char *cmd, netbuf *nControl)
 {
-    char buf[256];
+    char buf[TMP_BUFSIZ];
     
     if ((strlen(cmd) + 7) > sizeof(buf))
         return 0;
@@ -1010,7 +1120,7 @@ GLOBALDEF int FtpSysType(char *buf, int max, netbuf *nControl)
  */
 GLOBALDEF int FtpMkdir(const char *path, netbuf *nControl)
 {
-    char buf[256];
+    char buf[TMP_BUFSIZ];
 
     if ((strlen(path) + 6) > sizeof(buf))
         return 0;
@@ -1027,7 +1137,7 @@ GLOBALDEF int FtpMkdir(const char *path, netbuf *nControl)
  */
 GLOBALDEF int FtpChdir(const char *path, netbuf *nControl)
 {
-    char buf[256];
+    char buf[TMP_BUFSIZ];
 
     if ((strlen(path) + 6) > sizeof(buf))
         return 0;
@@ -1056,7 +1166,7 @@ GLOBALDEF int FtpCDUp(netbuf *nControl)
  */
 GLOBALDEF int FtpRmdir(const char *path, netbuf *nControl)
 {
-    char buf[256];
+    char buf[TMP_BUFSIZ];
 
     if ((strlen(path) + 6) > sizeof(buf))
         return 0;
@@ -1104,9 +1214,12 @@ static int FtpXfer(const char *localfile, const char *path,
 
     if (localfile != NULL)
     {
-	char ac[4] = "w";
+	char ac[4];
+	memset( ac, 0, sizeof(ac) );
 	if (typ == FTPLIB_FILE_WRITE)
 	    ac[0] = 'r';
+	else
+	    ac[0] = 'w';
 	if (mode == FTPLIB_IMAGE)
 	    ac[1] = 'b';
 	local = fopen(localfile, ac);
@@ -1120,27 +1233,40 @@ static int FtpXfer(const char *localfile, const char *path,
     if (local == NULL)
 	local = (typ == FTPLIB_FILE_WRITE) ? stdin : stdout;
     if (!FtpAccess(path, typ, mode, nControl, &nData))
+    {
+	if (localfile)
+	{
+	    fclose(local);
+	    if ( typ == FTPLIB_FILE_WRITE )
+		unlink(localfile);
+	}
 	return 0;
+    }
     dbuf = malloc(FTPLIB_BUFSIZ);
     if (typ == FTPLIB_FILE_WRITE)
     {
 	while ((l = fread(dbuf, 1, FTPLIB_BUFSIZ, local)) > 0)
+	{
 	    if ((c = FtpWrite(dbuf, l, nData)) < l)
 	    {
 		printf("short write: passed %d, wrote %d\n", l, c);
 		rv = 0;
 		break;
 	    }
+	}
     }
     else
     {
     	while ((l = FtpRead(dbuf, FTPLIB_BUFSIZ, nData)) > 0)
-	    if (fwrite(dbuf, 1, l, local) <= 0)
+	{
+	    if (fwrite(dbuf, 1, l, local) == 0)
 	    {
-		perror("localfile write");
+		if (ftplib_debug)
+		    perror("localfile write");
 		rv = 0;
 		break;
 	    }
+	}
     }
     free(dbuf);
     fflush(local);
@@ -1176,10 +1302,11 @@ GLOBALDEF int FtpDir(const char *outputfile, const char *path, netbuf *nControl)
  *
  * return 1 if successful, 0 otherwise
  */
-GLOBALDEF int FtpSize(const char *path, int *size, char mode, netbuf *nControl)
+GLOBALDEF int FtpSize(const char *path, unsigned int *size, char mode, netbuf *nControl)
 {
-    char cmd[256];
-    int resp,sz,rv=1;
+    char cmd[TMP_BUFSIZ];
+    int resp,rv=1;
+    unsigned int sz;
 
     if ((strlen(path) + 7) > sizeof(cmd))
         return 0;
@@ -1191,7 +1318,7 @@ GLOBALDEF int FtpSize(const char *path, int *size, char mode, netbuf *nControl)
 	rv = 0;
     else
     {
-	if (sscanf(nControl->response, "%d %d", &resp, &sz) == 2)
+	if (sscanf(nControl->response, "%d %u", &resp, &sz) == 2)
 	    *size = sz;
 	else
 	    rv = 0;
@@ -1199,6 +1326,37 @@ GLOBALDEF int FtpSize(const char *path, int *size, char mode, netbuf *nControl)
     return rv;
 }
 
+#if defined(__UINT64_MAX)
+/*
+ * FtpSizeLong - determine the size of a remote file
+ *
+ * return 1 if successful, 0 otherwise
+ */
+GLOBALDEF int FtpSizeLong(const char *path, fsz_t *size, char mode, netbuf *nControl)
+{
+    char cmd[TMP_BUFSIZ];
+    int resp,rv=1;
+    fsz_t sz;
+
+    if ((strlen(path) + 7) > sizeof(cmd))
+        return 0;
+    sprintf(cmd, "TYPE %c", mode);
+    if (!FtpSendCmd(cmd, '2', nControl))
+	return 0;
+    sprintf(cmd,"SIZE %s",path);
+    if (!FtpSendCmd(cmd,'2',nControl))
+	rv = 0;
+    else
+    {
+	if (sscanf(nControl->response, "%d %" PRIu64 "", &resp, &sz) == 2)
+	    *size = sz;
+	else
+	    rv = 0;
+    }   
+    return rv;
+}
+#endif
+
 /*
  * FtpModDate - determine the modification date of a remote file
  *
@@ -1206,7 +1364,7 @@ GLOBALDEF int FtpSize(const char *path, int *size, char mode, netbuf *nControl)
  */
 GLOBALDEF int FtpModDate(const char *path, char *dt, int max, netbuf *nControl)
 {
-    char buf[256];
+    char buf[TMP_BUFSIZ];
     int rv = 1;
 
     if ((strlen(path) + 7) > sizeof(buf))
@@ -1248,7 +1406,7 @@ GLOBALDEF int FtpPut(const char *inputfile, const char *path, char mode,
  */
 GLOBALDEF int FtpRename(const char *src, const char *dst, netbuf *nControl)
 {
-    char cmd[256];
+    char cmd[TMP_BUFSIZ];
 
     if (((strlen(src) + 7) > sizeof(cmd)) ||
         ((strlen(dst) + 7) > sizeof(cmd)))
@@ -1269,7 +1427,7 @@ GLOBALDEF int FtpRename(const char *src, const char *dst, netbuf *nControl)
  */
 GLOBALDEF int FtpDelete(const char *fnm, netbuf *nControl)
 {
-    char cmd[256];
+    char cmd[TMP_BUFSIZ];
 
     if ((strlen(fnm) + 7) > sizeof(cmd))
         return 0;
